@@ -36,6 +36,10 @@
 #include "printlabeldlg.h"
 #include "mimesources.h"
 #include "zplutils.h"
+#include "measurements.h"
+#include "kbarcodesettings.h"
+#include "batchprinter.h"
+#include "textfields.h"
 
 // QT includes
 #include <qbuffer.h>
@@ -94,48 +98,19 @@
 #include <kstatusbar.h>
 #include <kstandarddirs.h>
 #include <ktempfile.h>
+#include <dcopclient.h>
 
 #define STATUS_ID_SIZE 100
 #define STATUS_ID_TEMPLATE 101
 #define STATUS_ID_MOUSE 102
 
-#define ID_ITEM_GROUP 6000
-#define ID_ITEM_FILENAME 6001
-#define ID_ITEM_DATE 6002
-#define ID_ITEM_ARTICLE_NO 6003
-#define ID_ITEM_ARTICLE_DESCRIPTION 6004
-#define ID_ITEM_BARCODE_NO 6005
-#define ID_ITEM_FIELD_0 6006
-#define ID_ITEM_FIELD_1 6007
-#define ID_ITEM_FIELD_2 6008
-#define ID_ITEM_FIELD_3 6009
-#define ID_ITEM_FIELD_4 6010
-#define ID_ITEM_FIELD_5 6011
-#define ID_ITEM_FIELD_6 6012
-#define ID_ITEM_FIELD_7 6013
-#define ID_ITEM_FIELD_8 6014
-#define ID_ITEM_FIELD_9 6015
-#define ID_ITEM_CUSTOMER_NAME 6016
-#define ID_ITEM_CUSTOMER_NO 6017
-#define ID_ITEM_LINE_0 6018
-#define ID_ITEM_LINE_1 6019
-#define ID_ITEM_LINE_2 6020
-#define ID_ITEM_LINE_3 6021
-#define ID_ITEM_LINE_4 6022
-#define ID_ITEM_LINE_5 6023
-#define ID_ITEM_LINE_6 6024
-#define ID_ITEM_LINE_7 6025
-#define ID_ITEM_LINE_8 6026
-#define ID_ITEM_LINE_9 6027
-#define ID_ITEM_ENCODING_TYPE 6028
-#define ID_RESOLUTION 6029
-
-#define ID_LOCK_ITEM 6050;
+#define ID_LOCK_ITEM 8000;
 
 using namespace KABC;
 
 LabelEditor::LabelEditor( QWidget *parent, QString _filename, const char *name, WFlags f )
-    : DSMainWindow( parent, name, f )
+    : DCOPObject( "LabelEditor" ),
+      DSMainWindow( parent, name, f )
 {
     description = "";
     d = new Definition();
@@ -169,17 +144,21 @@ LabelEditor::LabelEditor( QWidget *parent, QString _filename, const char *name,
     connect( cv, SIGNAL( doubleClickedItem(QCanvasItem*) ), this, SLOT( doubleClickedItem(QCanvasItem*) ) );
     connect( cv, SIGNAL( showContextMenu(QCanvasItem*, QPoint) ), this, SLOT( showContextMenu(QCanvasItem*,QPoint) ) );
     connect( cv, SIGNAL( movedSomething() ), this, SLOT( setEdited() ) );
+    connect( cv, SIGNAL( selectionChanged() ), this, SLOT( enableControls() ) );
+    connect( KBarcodeSettings::getInstance(), SIGNAL( updateGrid( int ) ), cv, SLOT( updateGUI() ) );
     connect( kapp, SIGNAL( aboutToQuit() ), this, SLOT( saveConfig() ) );
 
     connect( history, SIGNAL( commandExecuted() ), cv, SLOT( updateGUI() ) );
     connect( history, SIGNAL( commandExecuted() ), this, SLOT( setEdited() ) );
 
     if( !_filename.isEmpty() )
-        load( _filename );
+        openUrl( _filename );
     else {
-        if( newdlg )
+        if( KBarcodeSettings::getInstance()->newDialog() )
             startupDlg( CreateNewLabel );
     }
+    
+    enableControls();
 }
 
 LabelEditor::~LabelEditor()
@@ -193,10 +172,8 @@ void LabelEditor::loadConfig()
     recentAct->loadEntries( config, "RecentFiles" );
 
     config->setGroup("LabelEditor");
-    c->setGrid( gridsize );
-    c->setGrid( gridcolor );
-    history->setUndoLimit( undolimit );
-    history->setRedoLimit( undolimit );
+    history->setUndoLimit( KBarcodeSettings::getInstance()->undoLimit() );
+    history->setRedoLimit( KBarcodeSettings::getInstance()->undoLimit() );
 
     gridAct->setChecked( config->readBoolEntry("gridenabled", false ) );
     toggleGrid();
@@ -267,9 +244,23 @@ bool LabelEditor::save( QString name )
         QFile::remove( name );
         
     QFile f( name );
-    if ( !f.open( IO_ReadWrite ) )
+    if ( !f.open( IO_WriteOnly ) )
         return false;
 
+    save( &f );
+
+    filename = name;
+    history->documentSaved();
+    m_edited = false;
+
+    enableActions();    
+    setCaption( filename, false );
+        
+    return true;
+}
+
+void LabelEditor::save( QIODevice* device )
+{
     QDomDocument doc( "KBarcodeLabel" );
     QDomElement root = doc.createElement( "kbarcode" );
     doc.appendChild( root );
@@ -314,7 +305,7 @@ bool LabelEditor::save( QString name )
 
             if(!buf.open( IO_WriteOnly ))
                 qDebug("Can't open buffer!");
-            
+
             (void)iio->write();
 
             buf.close();
@@ -322,7 +313,7 @@ bool LabelEditor::save( QString name )
             tag.appendChild( doc.createCDATASection( QString( buf.buffer() ) ) );
             root.appendChild( tag );
         } else if( list[i]->rtti() == TEXT_RTTI ) {
-            // Text 
+            // Text
             CanvasText* mytext = (CanvasText*)list[i];
             QDomElement tag = doc.createElement( "textfield" );
             writeXMLRect( &tag, QRect( (int)mytext->x(), (int)mytext->y(), mytext->width(), mytext->height() ), cv );
@@ -339,7 +330,7 @@ bool LabelEditor::save( QString name )
             DrawingRect* rc = (DrawingRect*)list[i];
             QDomElement tag = doc.createElement( "rect" );
             writeXMLRect( &tag, QRect( (int)rc->x(), (int)rc->y(), rc->width(), rc->height() ), cv );
-            
+
             tag.setAttribute( "z", rc->z() );
             tag.setAttribute( "circle", rc->isCircle() );
             tag.setAttribute( "borderwidth", rc->borderWidth() );
@@ -347,7 +338,7 @@ bool LabelEditor::save( QString name )
             writeXMLColor( &tag, "color", rc->color() );
             writeXMLColor( &tag, "bordercolor", rc->borderColor() );
 
-            root.appendChild( tag );        
+            root.appendChild( tag );
         } else if( list[i]->rtti() == LINE_RTTI ) {
             MyCanvasLine* cl = (MyCanvasLine*)list[i];
             QDomElement tag = doc.createElement( "line" );
@@ -355,7 +346,7 @@ bool LabelEditor::save( QString name )
             QPoint translation = cv->getTranslation();
             QPoint s = cl->startPoint() - translation;
             QPoint e = cl->endPoint() - translation;
-                                                
+
             tag.setAttribute( "x1", s.x() );
             tag.setAttribute( "y1", s.y() );
             tag.setAttribute( "x2", e.x() );
@@ -363,44 +354,35 @@ bool LabelEditor::save( QString name )
 
             tag.setAttribute( "z", cl->z() );
             tag.setAttribute( "width", cl->pen().width() );
-            tag.setAttribute( "style", cl->pen().style() ); 
+            tag.setAttribute( "style", cl->pen().style() );
             writeXMLColor( &tag, "color", cl->pen().color() );
 
-            root.appendChild( tag );            
+            root.appendChild( tag );
         }
 
     QCString xml = doc.toCString();
-    f.writeBlock( xml, xml.length() );
-    f.close();
-
-    filename = name;
-    history->documentSaved();
-    m_edited = false;
-
-    enableActions();    
-    setCaption( filename, false );
-        
-    return true;
+    device->writeBlock( xml, xml.length() );
+    device->close();
 }
 
-bool LabelEditor::load()
+bool LabelEditor::open()
 {
     QString name = KFileDialog::getOpenFileName ( NULL, "*.kbarcode", this, i18n("Select Label") );
     if(name.isEmpty()) return false;
 
-    return load( name );    
+    return openUrl( name );
 }
 
-bool LabelEditor::load( QString name )
+bool LabelEditor::openUrl( const QString & url )
 {
-    if( name.isEmpty() ) {
-        return load();
+    if( url.isEmpty() ) {
+        return open();
     }
 
-    filename = name;
+    filename = url;
     setCaption( filename, false );
-    
-    QFile f( name );
+
+    QFile f( filename );
     if ( !f.open( IO_ReadOnly ) )
         return false;
 
@@ -515,9 +497,9 @@ bool LabelEditor::load( QString name )
         n = n.nextSibling();
     }
 
-    KURL url;
-    url.setPath( filename );
-    recentAct->addURL( url );
+    KURL murl;
+    murl.setPath( filename );
+    recentAct->addURL( murl );
 
     enableActions();
     cv->repaintContents( true );
@@ -533,9 +515,15 @@ bool LabelEditor::newLabel()
         return false;
     }
 
-    d->setId( nl->labelId() );
-    clearLabel();
-    cv->setDefinition( d );
+    closeLabel();
+    
+    if( !nl->empty() )
+    {
+        d->setId( nl->labelId() );
+        clearLabel();
+        cv->setDefinition( d );
+    }
+    
     delete nl;
 
     filename = "";
@@ -564,6 +552,7 @@ void LabelEditor::setupActions()
     exportZPLAct = new KAction( i18n("...as &IPL or ZPL"), 0, 0, this, SLOT(exportZPL() ), actionCollection() );
     descriptionAct = new KAction( i18n("&Change description..."), 0, 0, this, SLOT(changeDes()), actionCollection() );
     deleteAct = new KAction( i18n("&Delete Object"), QIconSet( BarIcon("editdelete") ), Key_Delete, cv, SLOT( deleteCurrent() ), actionCollection() );
+    editPropAct = new KAction( i18n("&Properties..."), 0, 0, this, SLOT(), actionCollection() );
     printAct = KStdAction::print( this, SLOT( print() ), actionCollection(), "print" );
     changeSizeAct = new KAction( i18n("&Change Label..."), 0, 0, this, SLOT( changeSize() ), actionCollection() );
     barcodeAct = new KAction( i18n("Insert &Barcode"), QIconSet( BarIcon("barcode") ), 0, this, SLOT( insertBarcode() ), actionCollection() );
@@ -604,9 +593,17 @@ void LabelEditor::setupActions()
     menuFields = new KActionMenu( i18n("Insert &Database Field"), QIconSet( BarIcon("text") ), this, "menuFields" );
     menuFields->setDelayed( false );
 
+    menuSystemFields = new KActionMenu( i18n("Insert System Field"), QIconSet( BarIcon("text") ), this, "menuSystemFields" );
+    menuSystemFields->setDelayed( false );
+
+    TextFields DataFields;
+    for(int idx= ID_ITEM_FIRST_SYSTEM; idx <= ID_ITEM_END; idx++ )
+        menuSystemFields->popupMenu()->insertItem( DataFields.TextFieldName(idx),idx );
+    
     KPopupMenu* mnuBasic = new KPopupMenu( this );
     mnuBasic->insertItem( i18n("Article Number"), ID_ITEM_ARTICLE_NO );
     mnuBasic->insertItem( i18n("Article Description"), ID_ITEM_ARTICLE_DESCRIPTION );
+    mnuBasic->insertItem( i18n("Barcode Number"), ID_ITEM_BARCODE_NO_BASIC );
     mnuBasic->insertSeparator();
     mnuBasic->insertItem( i18n("Field 0"), ID_ITEM_FIELD_0 );
     mnuBasic->insertItem( i18n("Field 1"), ID_ITEM_FIELD_1 );
@@ -624,6 +621,9 @@ void LabelEditor::setupActions()
     mnuCustomer->insertItem( i18n("Customer Number"), ID_ITEM_CUSTOMER_NO );
     
     KPopupMenu* mnuCustomerText = new KPopupMenu( this );
+    mnuCustomerText->insertItem( i18n("Article Number Customer"), ID_ITEM_ARTICLE_NO_CUSTOMER);
+    mnuCustomerText->insertItem( i18n("Barcode Number Customer"), ID_ITEM_BARCODE_NO_CUSTOMER);
+    mnuCustomerText->insertSeparator();
     mnuCustomerText->insertItem( i18n("Line 0"), ID_ITEM_LINE_0 );
     mnuCustomerText->insertItem( i18n("Line 1"), ID_ITEM_LINE_1 );
     mnuCustomerText->insertItem( i18n("Line 2"), ID_ITEM_LINE_2 );
@@ -646,8 +646,11 @@ void LabelEditor::setupActions()
     menuFields->popupMenu()->insertItem( i18n("Filename"), ID_ITEM_FILENAME );
     menuFields->popupMenu()->insertItem( i18n("Date"), ID_ITEM_DATE );
     menuFields->popupMenu()->insertItem( i18n("Resolution"), ID_RESOLUTION );
+    menuFields->popupMenu()->insertItem( i18n("Index"), ID_INDEX );
+    menuFields->popupMenu()->insertItem( i18n("Serial Number"), ID_SERIAL );
     
     connect( menuFields->popupMenu(), SIGNAL(activated(int)), this, SLOT(insertDataText(int)));
+    connect( menuSystemFields->popupMenu(), SIGNAL(activated(int)), this, SLOT(insertDataText(int)));
     connect( mnuBasic, SIGNAL(activated(int)), this, SLOT(insertDataText(int)));
     connect( mnuCustomer, SIGNAL(activated(int)), this, SLOT(insertDataText(int)));
     connect( mnuCustomerText, SIGNAL(activated(int)), this, SLOT(insertDataText(int)));
@@ -658,6 +661,7 @@ void LabelEditor::setupActions()
     pictureAct->plug( tools );
     textAct->plug( tools );
     menuFields->plug( tools );
+    menuSystemFields->plug( tools );
     lineAct->plug( tools );
     rectAct->plug( tools );
     circleAct->plug( tools );
@@ -714,11 +718,13 @@ void LabelEditor::setupActions()
     changeSizeAct->plug( editMenu );
     sep->plug( editMenu );
     deleteAct->plug( editMenu );
-
+    editPropAct->plug( editMenu );
+    
     barcodeAct->plug( insMenu );
     pictureAct->plug( insMenu );
     textAct->plug( insMenu );
     menuFields->plug( insMenu );
+    menuSystemFields->plug( insMenu );
     lineAct->plug( insMenu );
     rectAct->plug( insMenu );
     circleAct->plug( insMenu );
@@ -782,104 +788,10 @@ void LabelEditor::insertText()
 
 void LabelEditor::insertDataText( int index )
 {
-    QString data;
-    switch( index ) {
-        case ID_ITEM_GROUP:
-            data = "[group]";
-            break;
-        case ID_ITEM_FILENAME:
-            data = "[filename]";
-            break;
-        case ID_ITEM_DATE:
-            data = "[date]";
-            break;
-        case ID_ITEM_ARTICLE_NO:
-            data = "[article_no]";
-            break;
-        case ID_ITEM_ARTICLE_DESCRIPTION:
-            data = "[article_desc]";
-            break;
-        case ID_ITEM_BARCODE_NO:
-            data = "[barcode_no]";
-            break;
-        case ID_ITEM_FIELD_0:
-            data = "[field0]";
-            break;
-        case ID_ITEM_FIELD_1:
-            data = "[field1]";
-            break;
-        case ID_ITEM_FIELD_2:
-            data = "[field2]";
-            break;
-        case ID_ITEM_FIELD_3:
-            data = "[field3]";
-            break;
-        case ID_ITEM_FIELD_4:
-            data = "[field4]";
-            break;
-        case ID_ITEM_FIELD_5:
-            data = "[field5]";
-            break;
-        case ID_ITEM_FIELD_6:
-            data = "[field6]";
-            break;
-        case ID_ITEM_FIELD_7:
-            data = "[field7]";
-            break;
-        case ID_ITEM_FIELD_8:
-            data = "[field8]";
-            break;
-        case ID_ITEM_FIELD_9:
-            data = "[field9]";
-            break;
-        case ID_ITEM_CUSTOMER_NAME:
-            data = "[customer_name]";
-            break;
-        case ID_ITEM_CUSTOMER_NO:
-            data = "[customer_no]";
-            break;
-        case ID_ITEM_LINE_0:
-            data = "[line0]";
-            break;
-        case ID_ITEM_LINE_1:
-            data = "[line1]";
-            break;
-        case ID_ITEM_LINE_2:
-            data = "[line2]";
-            break;
-        case ID_ITEM_LINE_3:
-            data = "[line3]";
-            break;
-        case ID_ITEM_LINE_4:
-            data = "[line4]";
-            break;
-        case ID_ITEM_LINE_5:
-            data = "[line5]";
-            break;
-        case ID_ITEM_LINE_6:
-            data = "[line6]";
-            break;
-        case ID_ITEM_LINE_7:
-            data = "[line7]";
-            break;
-        case ID_ITEM_LINE_8:
-            data = "[line8]";
-             break;
-        case ID_ITEM_LINE_9:
-            data = "[line9]";
-            break;
-        case ID_ITEM_ENCODING_TYPE:
-            data = "[encoding_type]";
-            break;
-        case ID_RESOLUTION:
-            data = "[resolution]";
-            break;
-        default:
-            break;
-    };
+    TextFields data ;
 
-    if( !data.isEmpty() )
-        insertText( data );
+    if( !data.TextField(index).isEmpty() )
+        insertText( data.TextField(index) );
 }
 
 void LabelEditor::insertText( QString caption )
@@ -1017,9 +929,10 @@ void LabelEditor::changeSize()
 
 void LabelEditor::updateInfo()
 {
-    measurements* measure = d->getMeasurements();
-    statusBar()->changeItem( i18n("Size: ") + QString("%1mm x %2mm").arg(
-                 measure->width ).arg( measure->height  ), STATUS_ID_SIZE );
+    Measurements* measure = d->getMeasurements();
+    statusBar()->changeItem( i18n("Size: ") + QString("%1%2 x %3%4").arg(
+                 measure->width() ).arg( Measurements::system()
+                 ).arg( measure->height()  ).arg( Measurements::system() ), STATUS_ID_SIZE );
     statusBar()->changeItem( i18n("Label Template: ") + d->getProducer() + " - " + d->getType(), STATUS_ID_TEMPLATE );
 }
 
@@ -1150,10 +1063,9 @@ void LabelEditor::showContextMenu( QCanvasItem* item, QPoint pos )
     if( item->rtti() == TEXT_RTTI ){
         menu->insertItem( i18n("&Edit"), this, SLOT( changeText() ) );
         menu->insertSeparator();
-        menu->insertItem( i18n("&Properties"), this, SLOT( changePictureSettings() ) );
-    } else {
-        menu->insertItem( i18n("&Properties"), this, SLOT( doubleClickedCurrent() ) );
-    }
+    } 
+    
+    menu->insertItem( i18n("&Properties"), this, SLOT( doubleClickedCurrent() ) );
     
     menu->popup( pos );
 }
@@ -1192,85 +1104,38 @@ void LabelEditor::print()
     if( pld.exec() != QDialog::Accepted )
         return;
 
-    int quantity = pld.labels();
-    int move = pld.position();
     PrinterSettings::getInstance()->getData()->border = pld.border();
-    move--;
-
-    bool border = pld.border();
 
     KPrinter* printer = PrinterSettings::getInstance()->setupPrinter( this );
     if( !printer )
         return;
 
-    QString oldname = filename;
-    KTempFile tmp( QString::null, ".kbarcode" );
 
-    save( tmp.name() );
-    filename = oldname;
-    setCaption( filename, isChanged() );
-    tmp.file()->close();
+    QBuffer buffer;
+    if( !buffer.open( IO_WriteOnly ) )
+        return;
 
-    Label label( d, tmp.file(), tmp.name(), printer, QString::null, QString::null, QString::null );
-    tmp.unlink();
+    save( &buffer );
 
-    measurements* printmeasure = d->getMeasurements( printer );
-               
-    int c_h = 0, c_w = 0;
-    double curw = printmeasure->gap_left, curh = printmeasure->gap_top;
+    BatchPrinter batch( printer, this );
+    batch.setBuffer( &buffer );
+    batch.setMove( pld.position() );
+    batch.setSerial( QString::null, 1 );
+    batch.setName( filename );
+    batch.setDefinition( d );
+    batch.setCustomer( QString::null );
+    batch.setEvents( false );
 
-    QProgressDialog* progress = new QProgressDialog( i18n("Printing..."), i18n("&Cancel"), quantity,this );
-    progress->setMinimumDuration( 0 );
-    progress->setProgress( 0 );
-    QPainter painter( printer );
-    
-    QPaintDeviceMetrics pdm( painter.device() );
-    int pageh = pdm.height();   // in pixel
+    QValueList<BatchPrinter::data>* list = new QValueList<BatchPrinter::data>;
+    BatchPrinter::data m_data;
+    m_data.number = pld.labels();
+    m_data.article_no = QString::null;
+    m_data.group = QString::null;
+    list->append( m_data );
 
-    for( int i = 0; i < quantity + move; i++ ) {
+    batch.setData( list );
+    batch.start();
 
-        if( c_h >= printmeasure->num_h ) {
-            c_h = 0;
-            curw = printmeasure->gap_left;
-            curh += printmeasure->height + printmeasure->gap_v;
-            c_w++;
-        }
-
-        if( curh + printmeasure->height > pageh || c_w == printmeasure->num_v ) {
-            label.setPage( label.page() + 1 );
-            printer->newPage();
-                        
-            curh = printmeasure->gap_top;
-            curw = printmeasure->gap_left;
-            c_w = 0;
-        }
-
-        if( i >= move ) {
-            painter.setClipRect( (int)curw, (int)curh, (int)printmeasure->width, (int)printmeasure->height );
-            label.setIndex( i  - move );
-            painter.drawPicture( QPoint( (int)curw, (int)curh ), *label.picture() );
-            painter.setClipping( false );
-            if( border ) {
-                // Draw borders around labels
-                painter.setPen( QPen( Qt::black, 1 ) );
-                painter.drawRect( (int)curw, (int)curh, (int)printmeasure->width, (int)printmeasure->height );
-            }
-        }
-
-        curw += printmeasure->width + printmeasure->gap_h;
-        c_h++;
-
-        kapp->processEvents( 0 );
-        progress->setProgress( i );
-        if( progress->wasCancelled() ) {                        
-            printer->abort();
-            break;
-        }
-    }
-
-    painter.end();
-    delete printmeasure;
-    delete progress;
     delete printer;
 }
 
@@ -1377,18 +1242,14 @@ const QString LabelEditor::fileName() const
 
 void LabelEditor::preview()
 {
-    QString oldname = filename;
-    KTempFile tmp( QString::null, ".kbarcode" );
+    QBuffer buffer;
+    if( !buffer.open( IO_WriteOnly ) )
+        return;
 
-    save( tmp.name() );
-    filename = oldname;    
-    setCaption( filename, isChanged() );
-                
-    PreviewDialog* pd = new PreviewDialog( tmp.file(), d, fileName(), this );
+    save( &buffer );
+
+    PreviewDialog* pd = new PreviewDialog( &buffer, d, fileName(), this );
     pd->exec();
-
-    tmp.file()->close();
-    tmp.unlink();
 }
 
 void LabelEditor::toggleGrid()
@@ -1531,7 +1392,7 @@ void LabelEditor::paste()
 void LabelEditor::startEditor()
 {
     if( isChanged() ) {
-        LabelEditor* lb = new LabelEditor();
+        LabelEditor* lb = new LabelEditor( 0, QString::null, "LabelEditorWindow" );
         lb->startupDlg( CreateNewLabel, "" );
     } else
         newLabel();
@@ -1551,23 +1412,23 @@ void LabelEditor::startLoadRecentEditor( const KURL& url )
     }
     
     if( isChanged() )
-        new LabelEditor( 0, url.path() );
+        new LabelEditor( 0, url.path(), "LabelEditorWindow" );
     else
-        load( url.path() );
+        openUrl( url.path() );
 }
 
 void LabelEditor::startLoadEditor()
 {
     if( isChanged() ) {
-        LabelEditor* lb = new LabelEditor();
+        LabelEditor* lb = new LabelEditor( 0, QString::null, "LabelEditorWindow" );
         lb->startupDlg( LoadLabel, "" );
     } else
-        load();
+        open();
 }
 
 void LabelEditor::batchPrint()
 {
-    new LabelPrinter( "" );
+    new LabelPrinter( QString::null, 0, "BatchPrintingWindow" );
 }
 
 void LabelEditor::closeEvent( QCloseEvent* e )
@@ -1616,7 +1477,7 @@ bool LabelEditor::startupDlg( int mode, QString f )
             return false;
         }
     } else {
-        if(!load(f)) {
+        if(!openUrl(f)) {
             close();
             return false;
         }
@@ -1649,6 +1510,9 @@ void LabelEditor::setEdited()
 
 void LabelEditor::enableActions()
 {
+    editPropAct->setEnabled( cv->getActive() );
+    deleteAct->setEnabled( cv->getActive() );
+
     if( d->getId() == -1 ){
         // label closed
         deleteAct->setEnabled( false );
@@ -1662,6 +1526,7 @@ void LabelEditor::enableActions()
         spellAct->setEnabled( false );
         gridAct->setEnabled( false );
         menuFields->setEnabled( false );
+        menuSystemFields->setEnabled( false );
 
         saveAct->setEnabled( false );
         saveAsAct->setEnabled( false );
@@ -1687,6 +1552,7 @@ void LabelEditor::enableActions()
         spellAct->setEnabled( true );
         gridAct->setEnabled( true );
         menuFields->setEnabled( true );
+        menuSystemFields->setEnabled( true );
 
         saveAct->setEnabled( true );
         saveAsAct->setEnabled( true );
@@ -1701,6 +1567,8 @@ void LabelEditor::enableActions()
         copyAct->setEnabled( true );
         pasteAct->setEnabled( true );
     }
+    
+    enableControls();
 }
 
 void LabelEditor::launchAddressBook()
@@ -1708,6 +1576,10 @@ void LabelEditor::launchAddressBook()
     KRun::runCommand( "kaddressbook" );
 }
 
+void LabelEditor::enableControls()
+{
+}
+
 /*********************************************************/
 
 BarcodeSettingsDlg::BarcodeSettingsDlg( QWidget* parent,  const char* name )
@@ -1800,11 +1672,11 @@ PositionDialog::PositionDialog( QWidget* parent,  const char* name )
     QSpacerItem* spacer = new QSpacerItem( 0, 0, QSizePolicy::Minimum, QSizePolicy::Expanding );
 
     spinX = new KIntNumInput( plainPage() );
-    spinX->setLabel( i18n("X Position (mm):") );
+    spinX->setLabel( QString( i18n("X Position (%1):") ).arg( Measurements::system() ) );
     spinX->setRange( -10000, 10000, 1, false );
 
     spinY = new KIntNumInput( plainPage() );
-    spinY->setLabel( i18n("Y Position (mm):") );
+    spinY->setLabel( QString( i18n("Y Position (%1):") ).arg( Measurements::system() ) );
     spinY->setRange( -10000, 10000, 1, false );
 
     PSLayout->addWidget( spinX );
