@@ -22,6 +22,8 @@
 #include "printersettings.h"
 #include "printlabeldlg.h"
 #include "smalldialogs.h"
+#include "measurements.h"
+#include "batchprinter.h"
 
 // Other includes
 #include <stdlib.h>
@@ -31,6 +33,7 @@
 #include <qclipboard.h>
 #include <qcursor.h>
 #include <qgroupbox.h>
+#include <qhbox.h>
 #include <qlayout.h>
 #include <qpainter.h>
 #include <qpaintdevicemetrics.h>
@@ -43,7 +46,6 @@
 #include <kcombobox.h>
 #include <kfiledialog.h>
 #include <kiconloader.h>
-#include <kimageio.h>
 #include <klistview.h>
 #include <klineedit.h>
 #include <klineeditdlg.h>
@@ -58,11 +60,10 @@
 #define MNU_IMPORT_ID 9999
 
 // a small helper function
-QString removeQuote( QString text, QString quote )
-{
+QString removeQuote( QString text, QString quote ) {
     if( quote.isEmpty() )
         return text;
-        
+
     text = text.stripWhiteSpace();
 
     if( text.left( quote.length() ) == quote )
@@ -75,17 +76,18 @@ QString removeQuote( QString text, QString quote )
 }
 
 LabelPrinter::LabelPrinter( QString filename, QWidget *parent, const char *name, WFlags f )
-    : DSMainWindow(parent, name, f)
+        : DCOPObject("BatchPrinting"),
+          DSMainWindow(parent, name, f)
 {
     def = 0;
-    
+
     QGroupBox* w = new QGroupBox( this );
     w->setColumnLayout(0, Qt::Vertical );
+    w->setInsideSpacing( 40 );
+
     pageLayout = new QVBoxLayout( w->layout() );
     setCentralWidget( w );
 
-    buttonLayout = new QHBoxLayout( 0, 6, 6 );
-    dataLayout = new QVBoxLayout( 0, 6, 6 );
     QSpacerItem* spacer = new QSpacerItem( 20, 20, QSizePolicy::Expanding, QSizePolicy::Expanding );
 
     list = new KListView( w );
@@ -94,11 +96,13 @@ LabelPrinter::LabelPrinter( QString filename, QWidget *parent, const char *name,
     list->addColumn( i18n("Article Number") );
     list->addColumn( i18n("Group") );
     list->setAllColumnsShowFocus( true );
-    connect( list, SIGNAL(clicked(QListViewItem*,const QPoint &,int)),
+    connect( list, SIGNAL(doubleClicked(QListViewItem*,const QPoint &,int)),
              this, SLOT(changeItem(QListViewItem*,const QPoint &,int)));
 
+    compGroup = new KCompletion();
+
     QGridLayout* infoGrid = new QGridLayout();
-        
+
     info1 = new QLabel( w );
     info2 = new QLabel( w );
     info3 = new QLabel( w );
@@ -110,7 +114,7 @@ LabelPrinter::LabelPrinter( QString filename, QWidget *parent, const char *name,
     infoGrid->addWidget( info3, 0, 2 );
     infoGrid->addWidget( info4, 0, 3 );
     infoGrid->addMultiCellWidget( info5, 1, 1, 0, 3 );
-    
+
     buttonPrint = new KPushButton( w );
     buttonPrint->setText( i18n("&Print...") );
     buttonPrint->setIconSet( SmallIconSet("fileprint") );
@@ -121,82 +125,63 @@ LabelPrinter::LabelPrinter( QString filename, QWidget *parent, const char *name,
     buttonImages->setIconSet( SmallIconSet("image") );
     buttonImages->setEnabled( false );
 
-    gItem = new QGroupBox( i18n("Add Item"), w );
-    gItem->setColumnLayout( 0, Qt::Vertical );
-    gItem->layout()->setSpacing( 6 );
-    gItem->layout()->setMargin( 11 );
-    QGridLayout* grid = new QGridLayout( gItem->layout() );
+    QGridLayout* grid = new QGridLayout( 0, 6, 6 );
+    grid->addWidget( new QLabel( i18n( "Label:" ), w ), 0, 0 );
 
-    QLabel* label = new QLabel( gItem );
-    label->setText( i18n( "Customer name and no.:" ) );
-    grid->addWidget( label, 0, 0 );
+    urlrequester = new KURLRequester( w );
+    urlrequester->setMode( KFile::File | KFile::ExistingOnly );
+    urlrequester->setFilter( "*.kbarcode" );
+    grid->addMultiCellWidget( urlrequester, 0, 0, 1, 2 );
 
-    customerName = new KComboBox( false, gItem );
-    grid->addWidget( customerName, 0, 1 );
+    grid->addWidget( new QLabel( i18n( "Customer name and no.:" ), w ), 1, 0 );
 
-    customerId = new KComboBox( false, gItem );
-    grid->addWidget( customerId, 0, 2 );
+    customerName = new KComboBox( false, w );
+    grid->addWidget( customerName, 1, 1 );
+
+    customerId = new KComboBox( false, w );
+    grid->addWidget( customerId, 1, 2 );
 
     customerName->setEnabled( false );
     customerId->setEnabled( false );
-    
-    label = new QLabel( gItem );
-    label->setText( i18n( "Label:" ) );
-    grid->addWidget( label, 1, 0 );
-    
-    urlrequester = new KURLRequester( gItem );
-    urlrequester->setMode( KFile::File | KFile::ExistingOnly );
-    urlrequester->setFilter( "*.kbarcode" );
-    grid->addMultiCellWidget( urlrequester, 1, 1, 1, 2 );
 
-    label = new QLabel( gItem );
-    label->setText( i18n("Group:") );
-    grid->addWidget( label, 2, 0 );
+    grid->addWidget( new QLabel( i18n( "Serial start:" ), w ), 2, 0 );
 
-    groupName = new KLineEdit( gItem );
-    grid->addMultiCellWidget( groupName, 2, 2, 1, 2 );
+    serialStart = new KLineEdit( w );
+    grid->addWidget( serialStart, 2, 1 );
 
-    numberArticles = new KIntNumInput( 1, gItem );
-    numberArticles->setLabel( i18n( "Number of labels:" ), KNumInput::AlignLeft | KNumInput::AlignVCenter );
-    numberArticles->setRange( 1, 10000, 1, false );
-    grid->addWidget( numberArticles, 3, 0 );
+    serialInc = new KIntNumInput( 1, w );
+    serialInc->setLabel( i18n( "Serial increment:" ), KNumInput::AlignLeft | KNumInput::AlignVCenter );
+    serialInc->setRange( 1, 10000, 1, false );
+    grid->addWidget( serialInc, 2, 2 );
 
-    label = new QLabel( gItem );
-    label->setText( i18n( "Article number:" ) );
-    grid->addWidget( label, 3, 1 );
+    QGroupBox* hbox = new QGroupBox( 4, QGroupBox::Horizontal, w );
+    buttonAdd = new KPushButton( hbox );
+    buttonAdd->setText( i18n( "&Add..." ) );
 
-    articleId = new KLineEdit( gItem );
-    grid->addWidget( articleId, 3, 2 );
+    buttonEdit = new KPushButton( hbox );
+    buttonEdit->setText( i18n( "&Edit..." ) );
 
-    buttonAdd = new KPushButton( gItem );
-    buttonAdd->setText( i18n( "&Add" ) );
-
-    buttonRemove = new KPushButton( gItem );
+    buttonRemove = new KPushButton( hbox );
     buttonRemove->setText( i18n("&Remove" ) );
 
-    buttonRemoveAll = new KPushButton( gItem );
+    buttonRemoveAll = new KPushButton( hbox );
     buttonRemoveAll->setText( i18n("R&emove All") );
 
-    grid->addWidget( buttonAdd, 6, 0 );
-    grid->addWidget( buttonRemove, 6, 1 );
-    grid->addWidget( buttonRemoveAll, 6, 2 );
-    
-    dataLayout->addWidget( gItem );
-    dataLayout->addWidget( list );
-    dataLayout->setStretchFactor( list, 2 );
-    dataLayout->addLayout( infoGrid );
+    pageLayout->addLayout( grid );
+    pageLayout->addWidget( hbox );
+    pageLayout->addWidget( list );
+    pageLayout->setStretchFactor( list, 2 );
+    pageLayout->addLayout( infoGrid );
 
+    QHBoxLayout* buttonLayout = new QHBoxLayout( 0, 6, 6 );
     buttonLayout->addItem( spacer );
     buttonLayout->addWidget( buttonPrint );
     buttonLayout->addWidget( buttonImages );
 
-    pageLayout->addLayout( dataLayout );
     pageLayout->addLayout( buttonLayout );
-    pageLayout->setStretchFactor( dataLayout, 2 );
     pageLayout->addItem( spacer );
 
     description = "";
-    measure = 0;
 
     DSMainWindow::setupActions();
     setupActions();
@@ -204,6 +189,7 @@ LabelPrinter::LabelPrinter( QString filename, QWidget *parent, const char *name,
     setupSql();
 
     connect( buttonAdd, SIGNAL( clicked() ), this, SLOT( addItem() ) );
+    connect( buttonEdit, SIGNAL( clicked() ), this, SLOT( editItem() ) );
     connect( buttonRemove, SIGNAL( clicked() ), this, SLOT( removeItem() ) );
     connect( buttonRemoveAll, SIGNAL( clicked() ), list, SLOT( clear() ) );
     connect( buttonRemoveAll, SIGNAL( clicked() ), this, SLOT( updateInfo() ) );
@@ -218,7 +204,7 @@ LabelPrinter::LabelPrinter( QString filename, QWidget *parent, const char *name,
 
     if( !filename.isEmpty() )
         loadFromFile( filename );
-    
+
     updateInfo();
     show();
 }
@@ -226,19 +212,17 @@ LabelPrinter::LabelPrinter( QString filename, QWidget *parent, const char *name,
 LabelPrinter::~LabelPrinter()
 {
     DSMainWindow::saveConfig();
-    if( measure )
-        delete measure;
+    delete compGroup;
     if( def )
         delete def;
 }
 
-void LabelPrinter::setupActions()
-{
+void LabelPrinter::setupActions() {
     mnuImport = new KPopupMenu( this );
     mnuImport->insertItem( i18n("Import from File ..."), this, SLOT( loadFromFile() ) );
     mnuImport->insertItem( i18n("Import from Clipboard ..."), this, SLOT( loadFromClipboard() ) );
     mnuImport->insertItem( i18n("Import barcode_basic"), this, SLOT( addAllItems() ) );
-    
+
     menuBar()->insertItem( i18n("&Import"), mnuImport, MNU_IMPORT_ID, 1 );
     menuBar()->setItemEnabled( MNU_IMPORT_ID, false );
 }
@@ -248,7 +232,7 @@ void LabelPrinter::setupSql()
     SqlTables* tables = SqlTables::getInstance();
     if( !tables->isConnected() )
         return;
-        
+
     QSqlCursor cur( TABLE_CUSTOMER );
     cur.select();
     customerId->clear();
@@ -258,22 +242,12 @@ void LabelPrinter::setupSql()
         customerName->insertItem( cur.value("customer_name" ).toString() );
     }
 
-    KCompletion* comp = articleId->completionObject();
-    comp->clear();
-    QSqlQuery query( "select article_no from " TABLE_BASIC " order by article_no" );
-    QStringList slist;
-    while ( query.next() )
-        slist.append( query.value(0).toString() );
-
-    comp->setItems( slist );
-
-    menuBar()->setItemEnabled( MNU_IMPORT_ID, true );    
+    menuBar()->setItemEnabled( MNU_IMPORT_ID, true );
     customerName->setEnabled( true );
     customerId->setEnabled( true );
 }
 
-void LabelPrinter::setCustomerId( QString id )
-{
+void LabelPrinter::setCustomerId( const QString & id ) {
     for( int i = 0; i < customerId->count(); i++ )
         if( customerId->text( i ) == id ) {
             customerId->setCurrentItem( i );
@@ -282,63 +256,83 @@ void LabelPrinter::setCustomerId( QString id )
         }
 }
 
-void LabelPrinter::setLabelURL( QString url )
-{
+void LabelPrinter::setLabelURL( const QString & url ) {
     urlrequester->setURL( url );
 }
 
-bool LabelPrinter::existsArticle( QString article )
+bool LabelPrinter::existsArticle( const QString & article ) 
 {
     if( article.isEmpty() )
         return false;
-    
+
     QSqlQuery query( "select uid from barcode_basic where article_no='" + article + "'" );
     while ( query.next() )
         return true;
 
-    return false;       
+    return false;
 }
 
 void LabelPrinter::addItem()
 {
-    if( !articleId->text().isEmpty() && !existsArticle( articleId->text() ) ) {
-        KMessageBox::error( this, i18n("Please enter a valid article ID") );
-        return;
+    DSSmallDialogs::AddItemsDialog aid( this, "aid" );
+    aid.setGroupCompletion( compGroup );
+    connect( &aid, SIGNAL( add( const QString &, const QString &, int) ),
+             this, SLOT( addItem( const QString &, const QString &, int) ) );
+
+    aid.exec();
+}
+
+bool LabelPrinter::addItem( const QString & article, const QString & group, int count, bool msg )
+{
+    if( !article.isEmpty() && !existsArticle( article ) ) {
+        if( msg )
+            KMessageBox::error( this, i18n("Please enter a valid article ID") );
+        return false;
     }
 
     QString temp;
     temp.sprintf("%0*i", 5, list->childCount() + 1 );
-        
-    KListViewItem* item = new KListViewItem( list, temp, QString( "%1" ).arg( numberArticles->value() ),
-            articleId->text(), groupName->text() );
+
+    KListViewItem* item = new KListViewItem( list, temp, QString( "%1" ).arg( count ),
+                          article, group );
     list->insertItem( item );
 
     updateInfo();
-        
-    if( !groupName->text().isEmpty() ) {
-          bool isinside = false;
-          KCompletion* comp = groupName->completionObject();
-          QStringList slist = comp->items();
-          for( unsigned int i = 0; i < slist.count(); i++ )
-              if( slist[i] == groupName->text() ) {
-                  isinside = true;
-                  break;
-              }
 
-          if(!isinside)                
-              comp->addItem( groupName->text() );
-    }
-
-    articleId->setText("");
-    groupName->setText("");
+    addGroupCompletion( group );
+    return true;
 }
 
-void LabelPrinter::addAllItems()
+void LabelPrinter::addGroupCompletion( const QString & group )
 {
+    if( !group.isEmpty() ) {
+        bool isinside = false;
+        QStringList slist = compGroup->items();
+        for( unsigned int i = 0; i < slist.count(); i++ )
+            if( slist[i] == group ) {
+                isinside = true;
+                break;
+            }
+
+        if(!isinside)
+            compGroup->addItem( group );
+    }
+}
+
+const QStringList LabelPrinter::allItems() {
+    QStringList list;
+    QSqlQuery query("SELECT article_no FROM " TABLE_BASIC );
+    while( query.next() )
+        list.append( query.value( 0 ).toString() );
+
+    return list;
+}
+
+void LabelPrinter::addAllItems() {
     DSSmallDialogs::AddAllDialog* dlg = new DSSmallDialogs::AddAllDialog( this, "dlg" );
     if( dlg->exec() != QDialog::Accepted )
         return;
-        
+
     QString temp;
     QString group = dlg->groupName();
     int n = dlg->numberLabels();
@@ -348,67 +342,107 @@ void LabelPrinter::addAllItems()
         temp.sprintf("%0*i", 5, list->childCount() + 1 );
 
         KListViewItem* item = new KListViewItem( list, temp, QString::number( n ),
-                query.value( 0 ).toString(), group );
+                              query.value( 0 ).toString(), group );
         list->insertItem( item );
     }
-    
+
     updateInfo();
 }
 
-void LabelPrinter::removeItem()
+void LabelPrinter::editItem()
 {
+    QListViewItem* item = list->selectedItem();
+    if( item )
+        changeItem( item, QPoint(0,0), 0 );
+}
+
+void LabelPrinter::removeItem() {
     QListViewItem* item = list->firstChild();
     while( item ) {
         if( item->isSelected() ) {
             QListViewItem* it = item->nextSibling();
             delete item;
-           
+
             while( it ) {
                 int a = it->text( 0 ).toInt();
                 QString temp;
-                temp.sprintf("%0*i", 5, a - 1 );                
+                temp.sprintf("%0*i", 5, a - 1 );
                 it->setText( 0, temp );
-                it = it->nextSibling();                
+                it = it->nextSibling();
             }
-            
+
             break;
         } else
             item = item->nextSibling();
     }
 
-    updateInfo();    
+    updateInfo();
 }
 
-void LabelPrinter::print( bool immediately, QString prn )
+void LabelPrinter::fillWithData( BatchPrinter* batch )
 {
-    PrintLabelDlg pld( this, "pld" );
-    pld.setLabelsEnabled( false );
-    if( pld.exec() != QDialog::Accepted )
-        return;
+    // sort by group
+    list->setSorting( 3, true );
+    list->sort();
 
-    int move = pld.position();
-    PrinterSettings::getInstance()->getData()->border = pld.border();
+    QValueList<BatchPrinter::data>* dlist = new QValueList<BatchPrinter::data>;
+    QListViewItem* item = list->firstChild();
+    while( item ) {
+        BatchPrinter::data m_data;
+        m_data.number = item->text( 1 ).toInt();
+        m_data.article_no = item->text( 2 );
+        m_data.group = item->text( 3 );
 
+        dlist->append( m_data );
+        item = item->nextSibling();
+    };
 
-    KPrinter* printer = PrinterSettings::getInstance()->setupPrinter( this, immediately, prn );
+    batch->setData( dlist );
+    batch->setLabels( labels );
+}
 
-    if( printer ) {
+void LabelPrinter::print( bool immediately, const QString & printer ) {
+    int move = 0;
+    if( !immediately ) {
+        PrintLabelDlg pld( this, "pld" );
+        pld.setLabelsEnabled( false );
+        if( pld.exec() != QDialog::Accepted )
+            return;
+
+        move = pld.position();
+        PrinterSettings::getInstance()->getData()->border = pld.border();
+    }
+
+    KPrinter* prn = PrinterSettings::getInstance()->setupPrinter( this, immediately, printer );
+
+    if( prn ) {
         QBuffer* buffer = getBuffer();
 
         if( !buffer ) {
-            delete printer;
+            delete prn;
             return;
         }
 
-        doPrint( printer, buffer, move );
+
+        BatchPrinter batch( prn, this );
+        batch.setBuffer( buffer );
+        batch.setMove( move );
+        batch.setSerial( serialStart->text(), serialInc->value() );
+        batch.setName( urlrequester->url() );
+        batch.setDefinition( def );
+        batch.setCustomer( customerId->currentText() );
+
+        fillWithData( &batch );
+
+        batch.start();
+
         delete buffer;
-        delete printer;
+        delete prn;
         KApplication::setOverrideCursor( QCursor(Qt::ArrowCursor), true );
     }
 }
 
-QBuffer* LabelPrinter::getBuffer()
-{
+QBuffer* LabelPrinter::getBuffer() {
     QFile f( urlrequester->url() );
     if ( !f.open( IO_ReadOnly ) )
         return 0;
@@ -419,256 +453,51 @@ QBuffer* LabelPrinter::getBuffer()
     QBuffer* buffer = new QBuffer( byte );
     if( !buffer->open( IO_ReadOnly ) )
         return 0;
-    
+
     return buffer;
 }
 
-void LabelPrinter::checkForNewPage( double ph, KPrinter* printer, Label* label )
-{
-    if( ph > pageh || c_w == measure->num_v ) {
-        printer->newPage();
-        label->setPage( label->page() + 1 );
-        curh = measure->gap_top;
-        curw = measure->gap_left;
-        c_w = 0;
-    }
-}
-
-void LabelPrinter::drawBorders( double x, double y, double w, double h, QPainter* painter )
-{
-    // Draw borders around labels
-    if( PrinterSettings::getInstance()->getData()->border ) {
-        painter->setPen( QPen( Qt::black, 1 ) );
-        painter->drawRect( (int)x, (int)y, (int)w, (int)h );
-    }
-    
-}
-
-void LabelPrinter::changeLine( int num, double height )
-{
-    if( c_h >= num ) {
-        c_h = 0;
-        curw = measure->gap_left;
-        curh += height + measure->gap_v;
-        c_w++;
-    }
-}
-
-void LabelPrinter::proccessEvents( int lb, KPrinter* printer, QPainter* p, QString value, Label* label )
-{
-    if( lb == NEW_PAGE ) {
-        if( curh > measure->gap_top || curw > measure->gap_left) {
-            printer->newPage();
-            label->setPage( label->page() + 1 );
-            curh = measure->gap_top;
-            curw = measure->gap_left;
-            c_h = 0;
-            c_w = 0;
-        }
-    }
-
-    if( lb == LINE_BREAK )
-        changeLine( measure->num_h, measure->height );
-
-    if( lb == ARTICLE_GROUP_NO ) {
-        changeLine( measure->num_h, measure->height );
-        checkForNewPage( curh + measure->height, printer, label );
-
-        Label::getXLabel( curw, curh, measure->width, measure->height, p, lb, value );
-        drawBorders( curw, curh, measure->width, measure->height, p );
-
-        curw += measure->width + measure->gap_h;
-        c_h++;
-    }
-
-    printXLabel( lb, printer, p, label );
-}
-
-void LabelPrinter::printXLabel( int lb, KPrinter* printer, QPainter* p, Label* label )
-{
-    if( lb == LABEL_X ) {
-        changeLine( measure->num_h, measure->height );
-        checkForNewPage( curh + measure->height, printer, label );
-
-        Label::getXLabel( curw, curh, measure->width, measure->height, p, lb, "" );
-
-        drawBorders( curw, curh, measure->width, measure->height, p );
-
-        curw += measure->width + measure->gap_h;
-        c_h++;
-    }
-}
-
-void LabelPrinter::doPrint( KPrinter* printer, QBuffer* buffer, int move )
-{
-    QPainter p( printer );
-    QPaintDeviceMetrics pdm( p.device() );
-    // int pagew = pdm.width();    // in pixel
-    pageh = pdm.height();   // in pixel
-
-    curw = 0;
-    curh = 0;
-
-    QString labelname = urlrequester->url();
-
-    QProgressDialog* progress = new QProgressDialog( i18n("Printing..."), i18n("&Cancel"), labels+1,this );
-    progress->setMinimumDuration( 0 );
-    progress->setProgress( 0 );
-
-    // sort by group
-    list->setSorting( 3, true );
-    list->sort();
-    progress->setProgress( 1 );
-    
-    QListViewItem* item = list->firstChild();
-    c_h = c_w = 0;
-
-    labelprinterdata* lpdata = PrinterSettings::getInstance()->getData();
-    measure = def->getMeasurements( printer );
-
-    bool moved = false;
-    QString oldgroup = QString::null;
-    unsigned int index = 0; // counter gets increased for every label, but not for 
-                                         // labels like article_no etc. (those who are created
-                                         // by events.
-    while( item ) {
-        Label* l = new Label( def, buffer, labelname, printer, customerId->currentText(),
-                 item->text( 2 ), item->text( 3 ) );
-
-        if( !moved ) {
-            for( int mi = 0; mi < move-1; mi++ ) {
-                if( !curw )
-                    curw = measure->gap_left;
-
-                if( !curh )
-                    curh = measure->gap_top;
-
-                changeLine( measure->num_h, measure->height );                    
-
-                if( curh + measure->height > pageh || c_w == measure->num_v ) {
-                    c_w = 0;
-                    curh = measure->gap_top;
-                    curw = measure->gap_left;
-                }
-
-                curw += measure->width + measure->gap_h;
-                c_h++;
-            }
-            moved = true;
-        }
-
-                
-        bool newgroup = false;
-        if( oldgroup == QString::null )
-            newgroup = true;
-        else {
-            if( item->text( 3 ) != oldgroup )
-                newgroup = true;    
-        }
-        oldgroup = item->text( 3 );                        
-
-        if( !curw )
-            curw = measure->gap_left;
-
-        if( !curh )
-            curh = measure->gap_top;
-
-        if( newgroup ) {
-            proccessEvents( lpdata->groupEvent1, printer, &p, i18n("Group : ") + item->text( 3 ), l );
-            proccessEvents( lpdata->groupEvent2, printer, &p, i18n("Group : ") + item->text( 3 ), l );
-            proccessEvents( lpdata->groupEvent3, printer, &p, i18n("Group : ") + item->text( 3 ), l );
-            proccessEvents( lpdata->groupEvent4, printer, &p, i18n("Group : ") + item->text( 3 ), l );
-        }
-        proccessEvents( lpdata->articleEvent1, printer, &p, i18n("Article No. : ") + item->text( 2 ), l );
-        proccessEvents( lpdata->articleEvent2, printer, &p, i18n("Article No. : ") + item->text( 2 ), l );
-        proccessEvents( lpdata->articleEvent3, printer, &p, i18n("Article No. : ") + item->text( 2 ), l );
-        proccessEvents( lpdata->articleEvent4, printer, &p, i18n("Article No. : ") + item->text( 2 ), l );
-
-        for( int i = 0; i < item->text( 1 ).toInt(); i++ ) {
-            changeLine( measure->num_h, measure->height );
-            checkForNewPage( curh + measure->height, printer, l );
-
-            p.setClipRect( (int)curw, (int)curh, (int)measure->width, (int)measure->height );
-            l->setIndex( index );
-            p.drawPicture( (int)curw, (int)curh, *l->picture() );
-            index++;
-            p.setClipping( false );
-            
-            drawBorders( curw, curh, measure->width, measure->height, &p );
-            
-            curw += measure->width + measure->gap_h;
-            c_h++;
-
-            kapp->processEvents( 0 );
-            progress->setProgress( progress->progress() + 1 );
-            if( progress->wasCancelled() ) {
-                printer->abort();
-                delete l;
-                delete progress;
-                return;
-            }
-        }
-
-        delete l;                
-        item = item->nextSibling();
-    }
-
-    p.end();
-    delete progress;
-}
-
-void LabelPrinter::customerIdChanged( int index )
-{
+void LabelPrinter::customerIdChanged( int index ) {
     customerName->setCurrentItem( index );
     enableControls();
 }
 
-void LabelPrinter::customerNameChanged( int index )
-{
+void LabelPrinter::customerNameChanged( int index ) {
     customerId->setCurrentItem( index );
 }
 
-void LabelPrinter::loadFromFile()
-{
+void LabelPrinter::loadFromFile() {
     QString f = KFileDialog::getOpenFileName( 0, 0, this );
     if( !f.isEmpty() )
         loadFromFile( f );
 }
 
-void LabelPrinter::loadFromClipboard()
-{
+void LabelPrinter::loadFromClipboard() {
     QClipboard *cb = KApplication::clipboard();
     QString text = cb->text();
     loadData( 0, text );
 }
 
-void LabelPrinter::loadFromFile( QString f )
-{
-    QFile file( f );
+void LabelPrinter::loadFromFile( const QString & url ) {
+    QFile file( url );
 
     if( !file.open( IO_ReadOnly ) ) {
-        qDebug("Unable to open file: " + f );
+        qDebug("Unable to open file: " + url );
         return;
     }
 
     loadData( &file );
-    
+
     file.close();
 }
 
-void LabelPrinter::loadData( QFile* file, const QString & data )
-{
+void LabelPrinter::loadData( QFile* file, const QString & data ) {
     labelprinterdata* lpdata = PrinterSettings::getInstance()->getData();
     if( lpdata->separator.isEmpty() ) {
         KMessageBox::sorry( this, i18n("Separator is empty. Please set it to a value.") );
         return;
     }
 
-    if( lpdata->comment.isEmpty() ) {
-        KMessageBox::sorry( this, i18n("Comment is empty. Please set it to a value.") );
-        return;
-    }
-
     // new config entry!!!
     KConfig* config = kapp->config();
     config->setGroup("FileFormat");
@@ -683,12 +512,12 @@ void LabelPrinter::loadData( QFile* file, const QString & data )
     QString s;
     QStringList dropped;
     QStringList datalist;
-    
+
     if( !file ) {
         datalist = QStringList::split( "\n", data, false );
     }
 
-    unsigned int i = 0;    
+    unsigned int i = 0;
     while( 1 ) {
         if( file ) {
             if( file->readLine( s, 1000 ) == -1 )
@@ -696,13 +525,14 @@ void LabelPrinter::loadData( QFile* file, const QString & data )
         } else {
             if( i >= datalist.count() )
                 break;
-                
+
             s = datalist[i];
             i++;
         }
-            
+
         s = s.stripWhiteSpace();
-        if( s.left( lpdata->comment.length() ) == lpdata->comment && !lpdata->comment.isEmpty() )
+        if(!lpdata->comment.isEmpty() && 
+           s.left( lpdata->comment.length() ) == lpdata->comment   )
             /* Line is a comment */
             continue;
 
@@ -760,30 +590,11 @@ void LabelPrinter::loadData( QFile* file, const QString & data )
             }
         }
 
-        if( qint && existsArticle( article ) ) {
-            QString index;
-            index.sprintf("%0*i", 5, list->childCount()+1 );
-
-            new KListViewItem( list, index, quantity, article, group );
-
-            if( !group.isEmpty() ) {
-                bool inside = false;
-                KCompletion* comp = groupName->completionObject();
-                QStringList slist = comp->items();
-                for( unsigned int i = 0; i < slist.count(); i++ )
-                    if( slist[i] == groupName->text() ) {
-                        inside = true;
-                        break;
-                    }
-                if( !inside )
-                    comp->addItem( groupName->text() );
-            }
-        } else
-            qint = false;
-
-        if( !qint )
-            dropped.append( quantity + lpdata->separator +  article + lpdata->separator + group );
-
+        if( qint ) // && existsArticle( article )
+        {    
+            if( !addItem( QString( article ), QString( group ), quantity.toInt(), false ) )
+                dropped.append( quantity + lpdata->separator +  article + lpdata->separator + group );
+        }
     }
 
     updateInfo();
@@ -793,19 +604,18 @@ void LabelPrinter::loadData( QFile* file, const QString & data )
         KMessageBox::informationList( this, i18n("<qt>The following items can not be added:" )+ "</qt>", dropped );
 #else
         KMessageBox::questionYesNoList( this, i18n("<qt>The following items can not be added:" )+ "</qt>", dropped );
-#endif            
+#endif
 }
 
-void LabelPrinter::updateInfo()
-{
+void LabelPrinter::updateInfo() {
     labels = 0;
     int articles = 0;
     int groups = 0;
     int pages = 0;
-    
+
     QStringList slist;
     QStringList glist;
-    
+
     QListViewItem* item = list->firstChild();
     while( item ) {
         labels += item->text( 1 ).toInt();
@@ -813,12 +623,12 @@ void LabelPrinter::updateInfo()
             glist.append( item->text( 3 ) );
             groups++;
         }
-                    
+
         if( !slist.contains( item->text( 2 ) ) ) {
             articles++;
             slist.append( item->text( 2 ) );
         }
-        
+
         item = item->nextSibling();
     }
 
@@ -828,11 +638,11 @@ void LabelPrinter::updateInfo()
     int num_v = 0;
 
     if( def ) {
-        measurements* m = def->getMeasurements();
-        num_h = m->num_h;
-        num_v = m->num_v;
+        Measurements* m = def->getMeasurements();
+        num_h = m->numH();
+        num_v = m->numV();
     }
-    
+
     int number_page = num_h * num_v;
     int fake = 0;
 
@@ -846,13 +656,15 @@ void LabelPrinter::updateInfo()
     info2->setText( QString(i18n("Different articles: %1")).arg( articles ) );
     info3->setText( QString(i18n("Different groups: %1")).arg( groups ) );
     info4->setText( QString(i18n("Pages: %1")).arg( pages ) );
-    info5->setText( i18n("Label description: ") + description );
+    if( !description.isEmpty() )
+        info5->setText( i18n("Label description: ") + description );
+    else
+        info5->setText( "" );
 
     enableControls();
 }
 
-void LabelPrinter::changeDescription()
-{
+void LabelPrinter::changeDescription() {
     QFile f( urlrequester->url() );
     if ( !f.open( IO_ReadOnly ) )
         return;
@@ -863,7 +675,7 @@ void LabelPrinter::changeDescription()
         delete doc;
         return;
     }
-    
+
     f.close();
     QDomNode n = doc->documentElement().firstChild();
     while( !n.isNull() ) {
@@ -878,7 +690,7 @@ void LabelPrinter::changeDescription()
                             description = e.text();
                         else if( e.tagName() == "id" )
                             def = readDefinition( &e, this );
-                            
+
                     node = node.nextSibling();
                 }
                 n = n.nextSibling();
@@ -892,126 +704,56 @@ void LabelPrinter::changeDescription()
     updateInfo();
 }
 
-void LabelPrinter::changeItem( QListViewItem* item, const QPoint &, int col )
+void LabelPrinter::changeItem( QListViewItem* item, const QPoint &, int )
 {
     if(!item)
         return;
-        
-    bool ok = false;
-    QString text = item->text( col );
-    QIntValidator v( 1, 100000, this );
-    QRegExpValidator vr( QRegExp("*", true, true ), 0 );
-    switch( col ) {
-        case 1:
-            text = KLineEditDlg::getText( "", item->text( col ),
-                   &ok, this, &v );
-            break;
-        case 2:
-            text = KLineEditDlg::getText( "", item->text( col ),
-                   &ok, this );
 
-            if( !text.isEmpty() )
-                if( !existsArticle( text ) ) {
-                    KMessageBox::error( this, i18n("Please enter a valid article ID") );
-                    ok = false;
-                }
-            break;
-        case 3:
-            text = KLineEditDlg::getText( "", item->text( col ),
-                   &ok, this, &vr );
-            break;
-        default:
-            break;
-    };
+    DSSmallDialogs::AddItemsDialog aid( item->text( 2 ), item->text( 3 ),
+                                        item->text( 1 ).toInt(), this, "aid" );
+    aid.setGroupCompletion( compGroup );
 
-    if( ok ) {
-        item->setText( col, text );
+    if( aid.exec() == QDialog::Accepted ) {
+        item->setText( 1, QString::number( aid.count() ) );
+        item->setText( 2, aid.articleNo() );
+        item->setText( 3, aid.groupName() );
+        addGroupCompletion( aid.groupName() );
         updateInfo();
     }
 }
 
 void LabelPrinter::enableControls()
 {
-    bool e = (!urlrequester->url().isEmpty() && def && list->childCount() && def->getId() > -1 ); 
+    bool e = (!urlrequester->url().isEmpty() && def && list->childCount() && def->getId() > -1 );
 
     buttonPrint->setEnabled( e );
     buttonImages->setEnabled( e );
-    
-    if( list->childCount() ) {
-        buttonRemove->setEnabled( true );
-        buttonRemoveAll->setEnabled( true );
-    } else {
-        buttonRemove->setEnabled( false );
-        buttonRemoveAll->setEnabled( false );
-    }
+
+    buttonRemove->setEnabled( list->childCount() );
+    buttonRemoveAll->setEnabled(list->childCount() );
+    buttonEdit->setEnabled( list->childCount() );
 }
 
-void LabelPrinter::images()
-{
+void LabelPrinter::images() {
     QString dirname = KFileDialog::getExistingDirectory ( QString::null, this, i18n("Select output directory") );
     if( dirname.isEmpty() )
         return;
-        
-    QString labelname = urlrequester->url();
+
     QBuffer* buffer = getBuffer();
     if( !buffer )
         return;
-    
-    QListViewItem* item = list->firstChild();
-    
-    QProgressDialog* progress = new QProgressDialog( i18n("Creating Images..."), i18n("&Cancel"), labels+1,this );
-    progress->setMinimumDuration( 0 );
-    progress->setProgress( 0 );
 
-    measure = def->getMeasurements( this );
+    BatchPrinter batch( dirname, this );
+    batch.setBuffer( buffer );
+    batch.setMove( 0 );
+    batch.setSerial( serialStart->text(), serialInc->value() );
+    batch.setName( urlrequester->url() );
+    batch.setDefinition( def );
+    batch.setCustomer( customerId->currentText() );
 
-    KConfig* conf = kapp->config();
-    conf->setGroup("ImageCreation");
-    bool filenamearticle = conf->readBoolEntry( "FilenameArticle", "true" );
-    QString format = conf->readEntry( "FileFormat", "PNG" );
-   
-    QPainter p;
-    QPixmap pixmap( (int)measure->width, (int)measure->height );
-    while( item ) {
-        Label* l = new Label( def, buffer, labelname, this, customerId->currentText(),
-                 item->text( 2 ), item->text( 3 ) );
+    fillWithData( &batch );
 
-        QPicture* pic = l->picture();
-        pixmap.fill( Qt::white );
-        p.begin( &pixmap );
-        p.drawPicture( 0, 0, *pic );
-        p.end();
-        for( int i = 0; i < item->text( 1 ).toInt(); i++ ) {
-            QString filename = dirname + "/" +
-                               ( filenamearticle ? item->text( 2 ) : l->barcodeNo() )
-                               + QString("_%1.").arg( i ) + KImageIO::suffix( format );
-
-            unsigned int c = 0;
-            while( QFile::exists( filename ) ) {
-                filename += "." + QString::number( c );
-                c++;
-            }
-                        
-            pixmap.save( filename, format );
-
-            kapp->processEvents( 0 );
-            progress->setProgress( progress->progress() + 1 );
-            if( progress->wasCancelled() ) {
-                delete l;
-                delete progress;
-                return;
-            }
-        }
-
-        delete l;
-        item = item->nextSibling();
-    }
+    batch.startImages();
     delete buffer;
-    delete measure;
-    delete progress;
-
     KMessageBox::information( this, i18n("Images creation finished.") );
 }
-
-
-
