@@ -19,6 +19,8 @@
 #include "sqltables.h"
 #include "barcodecache.h"
 
+#include <stdio.h>
+
 // Qt includes
 #include <qdir.h>
 #include <qpainter.h>
@@ -55,8 +57,6 @@ bool barcodeData::operator==( const struct barcodeData d ) const {
     if( BarCode::hasFeature( type, PDF417BARCODE ) )
         b = ( b && pdf417 == d.pdf417 );
 
-    b = b && colors ==  d.colors;
-    
     return b;
 };
 
@@ -89,32 +89,27 @@ const QPixmap BarCode::pixmap()
     return p;
 }
 
-bool BarCode::createPixmap( QPixmap* target, int res )
+bool BarCode::createPixmap( QPixmap* target, int resx, int resy )
 {
-    KTempFile* output = 0;
-    KTempFile* input = new KTempFile( QString::null, ".bmp" );
+    KTempFile* output = new KTempFile( QString::null, ".ps" );;
+    output->file()->close();
+    
+    KTempFile* input = new KTempFile( QString::null, ".pbm" );
     input->file()->close();
 
     m_value = createSequence( barcode.value );
 
     if( BarCode::hasFeature( barcode.type, PDF417 ) ) {
-        output = new KTempFile( QString::null, ".pbm" );
-        output->file()->close();
         if(!createPdf417( output )) {
             cleanUp( output, input, target );
             return false;
         }
     } else if( BarCode::hasFeature( barcode.type, TBARCODE ) ) {
-        output = new KTempFile( QString::null, ".ps" );
-        output->file()->close();
         if(!createTBarcode( output )) {
             cleanUp( output, input, target );
             return false;
         }
     } else { // if( BarCode::hasFeature( barcode.type, GNU_BARCODE ) ) {
-        output = new KTempFile( QString::null, ".ps" );
-        output->file()->close();
-
         QString flag = barcode.text ? "" : "-n";
 
         KShellProcess proc;
@@ -138,15 +133,16 @@ bool BarCode::createPixmap( QPixmap* target, int res )
         return false;
     }
 
+    QSize s = getBoundingBox( output->name() );
+    double sw = (double)s.width()/72 * resx;
+    double sh = (double)s.height()/72 * resy;
+    
     KShellProcess proc2;
-    proc2 << "convert";
-    if( barcode.colors.invert )
-        proc2 << "-negate";
-//          << "-fill \"#FF0000\" -colorize 50" //-opaque \"#000000\""
-
-    proc2 << "-density" << QString("%1").arg(res*(double)barcode.scale)
-              << output->name() << input->name();
-
+    proc2 << "gs" << QString("-g%1x%2").arg(int(sw*(double)barcode.scale)).arg(int(sh*(double)barcode.scale))
+          << "-r" + QString::number( resx*(double)barcode.scale ) + "x" + QString::number( resy*(double)barcode.scale )
+          << "-sDEVICE=pbmraw" << "-sOutputFile=" + input->name() << "-sNOPAUSE"
+          << "-q " + output->name() << "-c showpage" << "-c quit";
+    
     proc2.start( KProcess::Block, KProcess::NoCommunication );
     proc2.resume();
 
@@ -155,8 +151,8 @@ bool BarCode::createPixmap( QPixmap* target, int res )
         return false;
     }
 
-    target->load( input->name(), "BMP" );
-    BarcodeCache::instance()->write( barcode, res, target, m_value );
+    target->load( input->name(), "PBM" );
+    BarcodeCache::instance()->write( barcode, resx, resy, target, m_value );
 
     input->unlink();
     output->unlink();
@@ -170,14 +166,15 @@ bool BarCode::createPixmap( QPixmap* target, int res )
 void BarCode::createBarcode( QPixmap* target, QPaintDevice* device )
 {
     QPaintDeviceMetrics pdm( device );
-    int res = pdm.logicalDpiY();
+    int resx = pdm.logicalDpiX();
+    int resy = pdm.logicalDpiY();
 
     QString value = createSequence( barcode.value );
-    QPixmap* cached = BarcodeCache::instance()->read( barcode, res, value );
+    QPixmap* cached = BarcodeCache::instance()->read( barcode, resx, resy, value );
 
     // no matching barcode found in cache
     if( !cached ) {
-        if( !createPixmap( target, res ) )
+        if( !createPixmap( target, resx, resy ) )
             return;
     } else {
         *target = *cached;
@@ -189,11 +186,13 @@ void BarCode::createBarcode( QPixmap* target, QPaintDevice* device )
         // we scale already here and not at the end,
         // so that the addMargin function does not get a scaled margin.
         QPaintDeviceMetrics pdm( KApplication::desktop() );
-        int screenres = pdm.logicalDpiY();
+        int screenresx = pdm.logicalDpiX();
+        int screenresy = pdm.logicalDpiY();
     
         QWMatrix m;
-        double scale = (res/screenres)*barcode.scale;
-        m.scale( scale, scale );
+        double scalex = (resx/screenresx)*barcode.scale;
+        double scaley = (resy/screenresy)*barcode.scale;
+        m.scale( scalex, scaley );
         *target = target->xForm( m );
     }
     *target = cut( target, barcode.cut );
@@ -216,7 +215,7 @@ bool BarCode::createPdf417( KTempFile* output )
 
     // ps does not work use pbm!
     KShellProcess proc;
-    proc << "pdf417_enc" << "-tpbm" << text.name() << output->name()
+    proc << "pdf417_enc" << "-tps" << text.name() << output->name()
          << barcode.pdf417.row
          << barcode.pdf417.col
          << barcode.pdf417.err;
@@ -302,8 +301,6 @@ void BarCode::fillDefault( barcodeData* data )
     data->xml.x = 0;
     data->xml.y = 0;
 
-    data->colors.invert = false;
-    
     data->sequence.enabled = false;
     data->sequence.mode = NUM;
     data->sequence.step = 1;
@@ -489,9 +486,35 @@ bool BarCode::hasFeature( const QString & type, unsigned int feature )
     return false;
 }
 
+QSize BarCode::getBoundingBox( const QString & filename )
+{ 
+    QSize s(0,0);
+    QFile f( filename );
+    if( !f.open( IO_ReadOnly ) )
+        return s;
+        
+    QString t;
+    while( f.readLine( t, 1000 ) != -1 )
+    {
+        if( t.startsWith( "%%BoundingBox:") )
+        {
+            int x = 0;
+            int y = 0;
+            int w = 0;
+            int h = 0;
+            t = t.right( t.length() - 14 );
+            sscanf( (const char*)t, "%d %d %d %d", &x, &y, &w, &h );
+            s = QSize( w, h );
+            break;
+        }
+    }
+
+    f.close();
+    return s;
+}
+
 bool BarCode::m_haveGnuBarcode = false;
 bool BarCode::m_havePdfEncode = false;
-bool BarCode::m_haveConvert = false;
 bool BarCode::m_haveTBarcode = false;
 
 barcodeFormat BarCode::fillStruct( const QString & name, const QString & text, const int feature )
@@ -505,7 +528,6 @@ barcodeFormat BarCode::fillStruct( const QString & name, const QString & text, c
 
 void BarCode::setHaveBarcode()
 {
-    m_haveConvert = !KStandardDirs::findExe( "convert" ).isNull();
     m_haveGnuBarcode = !KStandardDirs::findExe( "barcode" ).isNull();
     m_havePdfEncode = !KStandardDirs::findExe( "pdf417_enc" ).isNull();
     m_haveTBarcode = !KStandardDirs::findExe( "tbarcodeclient" ).isNull();
