@@ -18,6 +18,8 @@
 #include "mycanvasview.h"
 #include "commands.h"
 #include "definition.h"
+#include "measurements.h"
+#include "kbarcodesettings.h"
 
 // Qt includes
 #include <qcursor.h>
@@ -31,8 +33,6 @@
 MyCanvas::MyCanvas( QObject* parent, const char* name )
     : QCanvas( parent, name )
 {
-    m_grid_size = 30;
-    m_grid_color = Qt::gray;
     m_grid = false;
 
     setBackgroundColor( Qt::lightGray );
@@ -64,9 +64,9 @@ void MyCanvas::drawBackground( QPainter & painter, const QRect & clip )
         if( m_grid ) {
             painter.translate( m_rect.x(), m_rect.y() );
 
-            painter.setPen( QPen( m_grid_color, 0 ) ); 
-            for( int x = 0; x <= m_rect.width(); x += m_grid_size )
-                for( int y = 0; y <= m_rect.height(); y += m_grid_size ) {
+            painter.setPen( QPen( KBarcodeSettings::getInstance()->gridColor(), 0 ) );
+            for( int x = 0; x <= m_rect.width(); x += KBarcodeSettings::getInstance()->gridSize() )
+                for( int y = 0; y <= m_rect.height(); y += KBarcodeSettings::getInstance()->gridSize() ) {
 //                    if( clip.x() < x && clip.x() + clip.height() > x )
                         painter.drawLine( x, 0, x, m_rect.height() );
 
@@ -87,8 +87,13 @@ MyCanvasView::MyCanvasView( Definition* d, MyCanvas* c, QWidget* parent, const c
 
     rulerv = new KRuler( Qt::Vertical, this );
     rulerh = new KRuler( Qt::Horizontal, this );
-    rulerv->setRulerMetricStyle( KRuler::Millimetres );
-    rulerh->setRulerMetricStyle( KRuler::Millimetres );
+    if( Measurements::measurementSystem() == Measurements::Metric ) {
+        rulerv->setRulerMetricStyle( KRuler::Millimetres );
+        rulerh->setRulerMetricStyle( KRuler::Millimetres );
+    } else {
+        rulerv->setRulerMetricStyle( KRuler::Inch );
+        rulerh->setRulerMetricStyle( KRuler::Inch );
+    }
     rulerv->setMaxValue( 0 );
     rulerh->setMaxValue( 0 );
 
@@ -100,6 +105,16 @@ MyCanvasView::~MyCanvasView()
 {
 }
 
+void MyCanvasView::snapPoint(QPoint * point)
+{
+    // move item to next grid position
+    // TODO: align the top left to the grid not the current mouse position!
+    point->setX(point->x() + KBarcodeSettings::getInstance()->gridSize()/2);
+    point->setY(point->y() + KBarcodeSettings::getInstance()->gridSize()/2);
+    point->setX(point->x() - point->x() % KBarcodeSettings::getInstance()->gridSize());
+    point->setY(point->y() - point->y() % KBarcodeSettings::getInstance()->gridSize());
+}
+
 void MyCanvasView::contentsMouseMoveEvent(QMouseEvent* e)
 {
     rulerh->slotNewValue( e->x() );
@@ -110,7 +125,8 @@ void MyCanvasView::contentsMouseMoveEvent(QMouseEvent* e)
         int x = (int)l.pixelToMm( e->x(), this, LabelUtils::DpiX );
         int y = (int)l.pixelToMm( e->y(), this, LabelUtils::DpiY );
         statusbar->changeItem( i18n("Position: ") +
-                        QString( "%1mm x %2mm" ).arg( x ).arg( y ), mouseid );
+                        QString( "%1%2 x %3%4" ).arg( x )
+                        .arg( Measurements::system() ).arg( y ).arg( Measurements::system()), mouseid );
     }
 
     updateCursor( e->pos() );
@@ -122,7 +138,7 @@ void MyCanvasView::contentsMouseMoveEvent(QMouseEvent* e)
     }
 
     bool shift_pressed = e->state() & Qt::ShiftButton;
-    
+
     MyCanvasRectangle* moving = getActive();
     if( moving ) {
         QPoint p = inverseWorldMatrix().map(e->pos());
@@ -131,9 +147,13 @@ void MyCanvasView::contentsMouseMoveEvent(QMouseEvent* e)
             QCanvasItemList list = getSelected();
             for( unsigned int i = 0; i < list.count(); i++ ) {
                 MyCanvasRectangle* moving = (MyCanvasRectangle*)list[i];
+                QPoint new_pt =QPoint(p.x() - delta_pt.x(),p.y() - delta_pt.y());
+                if( canv->grid() ) {
+                    snapPoint(&new_pt) ;
+                }
                 // Move the item
-                MoveCommand* mv = new MoveCommand( p.x() - moving_start.x(),
-                                                   p.y() - moving_start.y(), moving );
+                MoveCommand* mv = new MoveCommand( new_pt.x() - (int) moving->x(),
+                                                   new_pt.y() - (int) moving->y(), moving );
                 mv->execute();
                 getMoveCommand()->addCommand( mv );
             }
@@ -141,6 +161,10 @@ void MyCanvasView::contentsMouseMoveEvent(QMouseEvent* e)
             QPoint a, b;
             MyCanvasLine* line = (MyCanvasLine*)moving;
 
+            if( canv->grid() ) {
+                snapPoint(&p) ;
+            }
+            
             a = m_mode == LineLeft ? p : line->startPoint();
             b = m_mode == LineRight ? p : line->endPoint();
 
@@ -150,39 +174,37 @@ void MyCanvasView::contentsMouseMoveEvent(QMouseEvent* e)
             getMoveCommand()->addCommand( lmc );
             repaintContents( line->boundingRect() );
         } else {
-            // make the code more readable
-            int w = old.width();
-            int h = old.height();
-            int x = (int)old.x();
-            int y = (int)old.y();
-            int ox = old.x() + old.width();
-            int oy = old.y() + old.height();
+            
+            if( canv->grid() ) {
+                snapPoint(&p) ;
+            }
+            
             ResizeCommand* mv = new ResizeCommand( moving, shift_pressed );
             
             switch( m_mode ) {
                 case RightMiddle:
-                    mv->setRect( x, y, p.x() - ox + w, h );
+                    mv->setRect( (int) moving->x(), (int) moving->y(), p.x() - (int) moving->x(), moving->height() );
                     break;
                 case LeftMiddle:
-                    mv->setRect( p.x(), y, x - p.x() + w, h );
+                    mv->setRect( p.x(), (int) moving->y(), (int) moving->width()+ ((int) moving->x() - p.x()), (int) moving->height() );
                     break;
                 case BottomMiddle:
-                    mv->setRect( x, y, w, p.y() - y );
+                    mv->setRect( (int) moving->x(), (int) moving->y(), (int) moving->width(), p.y() - (int) moving->y());
                     break;
                 case TopMiddle:
-                    mv->setRect( x, p.y(), w, y - p.y() + h );
+                    mv->setRect( (int) moving->x(), p.y(), (int) moving->width(), (int) moving->height()+ ((int) moving->y() - p.y()));
                     break;
                 case BottomLeft:
-                    mv->setRect( p.x(), y, x - p.x() + w, p.y() - y );
+                    mv->setRect( p.x(), (int) moving->y(), (int) moving->width()+ ((int) moving->x() - p.x()), p.y() - (int) moving->y() );
                     break;
                 case BottomRight:
-                    mv->setRect( x, y, p.x() - ox + w, p.y() - oy + h );
+                    mv->setRect( (int) moving->x(), (int) moving->y(), p.x() - (int) moving->x(), p.y() - (int) moving->y() );
                     break;
                 case TopLeft:
-                    mv->setRect( p.x(), p.y(), x - p.x() + w, y - p.y() + h );
+                    mv->setRect( p.x(), p.y(), (int) moving->width()+ ((int) moving->x() - p.x()), (int) moving->height()+ ((int) moving->y() - p.y()));
                     break;
                 case TopRight:
-                    mv->setRect( x, p.y(), p.x() - x, y - p.y() + h );
+                    mv->setRect( (int) moving->x(), p.y(), p.x() - (int) moving->x(),(int) moving->height()+ ((int) moving->y() - p.y()) );
                     break;
                 default:
                     break;
@@ -211,6 +233,7 @@ void MyCanvasView::contentsMousePressEvent(QMouseEvent* e)
         moving_start = inverseWorldMatrix().map(e->pos());
         m_mode = updateCursor( e->pos() );
         old = getActive()->boundingRect();
+        delta_pt=QPoint(e->x() - old.x(),e->y() - old.y());
     }
 
     if( e->button() == Qt::RightButton && getActive() )
@@ -226,24 +249,8 @@ void MyCanvasView::contentsMouseReleaseEvent(QMouseEvent* e)
         history->addCommand( getMoveCommand(), false );
         m_commov = 0;
     }
-                
-    if( canv->grid() && (m_mode == Inside || m_mode == Barcode) ) {
-        QCanvasItemList list = getSelected();
-        for( unsigned int i = 0; i < list.count(); i++ ) {
-            MyCanvasRectangle* moving = (MyCanvasRectangle*)list[i];
-            
-            // move item to next grid position
-            int tx = ((int)moving->x() - translation.x()) % canv->getGrid();
-            int ty = ((int)moving->y() - translation.y()) % canv->getGrid();
-
-            tx = tx > canv->getGrid()/2  ? tx : tx * -1;
-            ty = ty > canv->getGrid()/2  ? ty : ty * -1;
-
-            MoveCommand* mv = new MoveCommand( tx, ty, moving );
-            history->addCommand( mv, true );
-        }
-    }
 
+    // TODO: check: is this needed
     if( m_mode == LineLeft || m_mode == LineRight )
         repaintContents();
 
@@ -389,16 +396,18 @@ void MyCanvasView::updateRuler()
     rulerv->setGeometry( 0, 20, 20, height() - 20 );
 
     if( def ) {
-        measurements* m = def->getMeasurements( this );;
-        canv->setRect( QRect( translation.x(), translation.y(), (int)m->width, (int)m->height) );
+        Measurements* m = def->getMeasurements();
+        canv->setRect( QRect( translation.x(), translation.y(), (int)m->width( this ), (int)m->height( this )) );
 
         rulerv->setMaxValue( height() );
         rulerh->setMaxValue( width() );
 
+
         QPaintDeviceMetrics pdm( this );
-        rulerh->setPixelPerMark( (1/25.4)* pdm.logicalDpiX() );
-        rulerv->setPixelPerMark( (1/25.4)* pdm.logicalDpiY() );
-        delete m;
+        if( Measurements::measurementSystem() == Measurements::Metric ) {
+            rulerh->setPixelPerMark( (1/ 25.4)* pdm.logicalDpiX() );
+            rulerv->setPixelPerMark( (1/25.4)* pdm.logicalDpiY() );
+        } 
     }
 
 }
@@ -408,11 +417,10 @@ void MyCanvasView::resizeEvent( QResizeEvent * r )
     setUpdatesEnabled( false );
     QPoint old = translation;
 
-    measurements* measure = def->getMeasurements( this );
+    Measurements* measure = def->getMeasurements();
     // make sure that there is some space around the label, therefore at 60 pixel
-    int w = ( this->width() - 2 > measure->width ? this->width() - 2 : int(measure->width + 60) );
-    int h = ( this->height() - 2 > measure->height ? this->height() - 2 : int(measure->height + 60) );
-    delete measure;
+    int w = ( this->width() - 2 > measure->width( this ) ? this->width() - 2 : int(measure->width( this ) + 60) );
+    int h = ( this->height() - 2 > measure->height( this ) ? this->height() - 2 : int(measure->height( this ) + 60) );
 
     canvas()->resize( w, h );
     QCanvasView::resizeEvent( r );
@@ -421,13 +429,13 @@ void MyCanvasView::resizeEvent( QResizeEvent * r )
     updateRuler();
 
     repaintContents();
-    
+
     old = translation - old;
     QCanvasItemList list = canvas()->allItems();
     for( unsigned int i = 0; i < list.count(); i++ )
         list[i]->moveBy( old.x(), old.y() );
-        
-    setUpdatesEnabled( true );        
+
+    setUpdatesEnabled( true );
 }
 
 void MyCanvasView::reposition()
@@ -440,13 +448,12 @@ void MyCanvasView::reposition()
      * As a result we have the correct width.
      */
 
-    measurements* measure = def->getMeasurements( this );
-    int x = int((width() - (measure->width)  ) / 2);
-    int y = int((height() - (measure->height) ) / 2);
-    delete measure;
+    Measurements* measure = def->getMeasurements();
+    int x = int((width() - (measure->width( this ))  ) / 2);
+    int y = int((height() - (measure->height( this )) ) / 2);
 
-    // move label 30 pixels away from top/left if necessary    
-    x = x > 0 ? x : 30; 
+    // move label 30 pixels away from top/left if necessary
+    x = x > 0 ? x : 30;
     y = y > 0 ? y : 30;
 
     translation = QPoint( x, y );
@@ -493,6 +500,7 @@ MyCanvasRectangle* MyCanvasView::getActive()
 
 void MyCanvasView::setActive( QCanvasItem* item, bool control )
 {
+    emit selectionChanged();
     QCanvasItemList list = canvas()->allItems();
     for( unsigned int i = 0; i < list.count(); i++ )
         list[i]->setActive( false );
