@@ -44,12 +44,22 @@ class MySqlDescription : public SqlDescription {
     public:
         virtual const QString initdb( QString ) const { return "mysql"; }
         virtual const QString autoIncrement() const { return "int(11) NOT NULL auto_increment"; }
+        virtual const QString showColumns( const QString & table ) const { return "SHOW COLUMNS FROM " + table; }
 };
 
 class PostgreSQLDescription : public SqlDescription {
     public:
         virtual const QString initdb( QString ) const { return "template1"; }
         virtual const QString autoIncrement() const { return "serial NOT NULL"; }
+        virtual const QString showColumns( const QString & table ) const {
+            return "select a.attname, pg_catalog.format_type(a.atttypid, a.atttypmod), "
+                   "a.attnotnull, a.atthasdef, a.attnum, substring(d.adsrc for 128) "
+                   "from pg_catalog.pg_attribute a "
+                   "inner join pg_catalog.pg_class c on c.oid = a.attrelid "
+                   "left join pg_catalog.pg_attrdef d on d.adrelid = c.oid and a.attnum = "
+                   "d.adnum "
+                   "where a.attnum > 0 and not a.attisdropped and c.relname ~ '^" + table + "$';";
+            }
 };
 
 /** should work for ODBC, too */
@@ -57,6 +67,7 @@ class SQLiteDescription : public SqlDescription {
     public:
         virtual const QString initdb( QString db = QString::null ) const { return db; }
         virtual const QString autoIncrement() const { return "serial NOT NULL"; }
+        virtual const QString showColumns( const QString & table ) const { return "SHOW COLUMNS FROM " + table; }
 };
 
 SqlTables::SqlTables( QObject* parent )
@@ -90,6 +101,10 @@ SqlTables* SqlTables::getInstance()
     return instance;
 }
 
+const SqlDescription* SqlTables::driver() const
+{
+    return drivers[sqldata->driver];
+}
 bool SqlTables::connectMySQL()
 {
     db = QSqlDatabase::addDatabase( sqldata->driver );
@@ -122,9 +137,9 @@ bool SqlTables::newTables()
 bool SqlTables::newTables( const QString & username, const QString & password, const QString & hostname, const QString & database, const QString & driver )
 {
     if( KMessageBox::warningContinueCancel( 0,
-                     i18n("We are going to re-create the tables '"
-                          TABLE_BASIC "', '" TABLE_CUSTOMER "', '"
-                          TABLE_CUSTOMER_TEXT "' and '" TABLE_LABEL_DEF "'")) ==  KMessageBox::Cancel )
+                     i18n("We are going to re-create the tables '") +
+                          QString( TABLE_BASIC "', '" TABLE_CUSTOMER "', '"
+                          TABLE_CUSTOMER_TEXT) +i18n("' and '") + QString(TABLE_LABEL_DEF "'")) ==  KMessageBox::Cancel )
         return false;
 
     QSqlDatabase*dbase = QSqlDatabase::addDatabase(driver, drivers[driver]->initdb( database ) );
@@ -142,7 +157,7 @@ bool SqlTables::newTables( const QString & username, const QString & password, c
 
         QSqlQuery firstquery( NULL, dbase );
         if( !found && !firstquery.exec("CREATE DATABASE " + database + ";")) {
-            if( KMessageBox::warningContinueCancel( 0, i18n("<qt>Can't create database "+ database + "<br>You can continue if the database exists already.</qt>")
+            if( KMessageBox::warningContinueCancel( 0, i18n("<qt>Can't create database ")+ database + i18n("<br>You can continue if the database exists already.</qt>")
                 + firstquery.lastError().databaseText() ) == KMessageBox::Cancel ) {
                 dbase->close();
                 QSqlDatabase::removeDatabase(drivers[driver]->initdb( database ));
@@ -241,7 +256,7 @@ bool SqlTables::newTables( const QString & username, const QString & password, c
 
         dbase->close();
         QSqlDatabase::removeDatabase( database );        
-        KMessageBox::information( 0, i18n("Created table "+database+" successfully!") );
+        KMessageBox::information( 0, i18n("Created table ")+database+i18n(" successfully!") );
     } else {
         dbase->close();
         QSqlDatabase::removeDatabase(drivers[driver]->initdb( database ));                
@@ -254,7 +269,7 @@ bool SqlTables::newTables( const QString & username, const QString & password, c
 
 void SqlTables::importLabelDef()
 {
-    if( KMessageBox::warningContinueCancel( 0, i18n("We are going to delete the complete table: " TABLE_LABEL_DEF ) ) ==
+    if( KMessageBox::warningContinueCancel( 0, i18n("We are going to delete the complete table: " ) + TABLE_LABEL_DEF ) ==
         KMessageBox::Cancel )
         return;
 
@@ -276,7 +291,7 @@ void SqlTables::importLabelDef()
 void SqlTables::importExampleData()
 {
     if( KMessageBox::warningContinueCancel( 0,
-        i18n("We are going to delete the complete tables: " TABLE_BASIC ", " TABLE_CUSTOMER ", " TABLE_CUSTOMER_TEXT ) ) ==
+        i18n("We are going to delete the complete tables: " ) +  QString( TABLE_BASIC ", " TABLE_CUSTOMER ", " TABLE_CUSTOMER_TEXT ) ) ==
         KMessageBox::Cancel )
         return;
 
